@@ -17,13 +17,14 @@ BASE_URL = "/api/%s/"
 
 REQUEST_METHODS = {
     "create_post": "POST",
+    "moderate_post": "POST",
     "get_forum_list": "GET",
     "get_forum_api_key": "GET",
     "get_thread_list": "GET",
     "get_num_posts": "GET",
     "get_thread_by_url": "GET",
     "get_thread_posts": "GET",
-    "thread_by_identiier": "POST",
+    "thread_by_identifier": "POST",
     "update_thread": "POST",
 }
 
@@ -42,6 +43,9 @@ class APIError(Exception):
 class DisqusService(object):
     _debug = False
 
+    def __init__(self, api_version="1.0"):
+        self.api_version = api_version
+
     def login(self, api_key):
         self.user_api_key = api_key
 
@@ -53,7 +57,8 @@ class DisqusService(object):
                     author_email,
                     parent_post=None,
                     created_at=None,
-                    author_url=None):
+                    author_url=None,
+                    ip_address=None):
         """
         Key: Forum Key
         Method: POST
@@ -68,8 +73,8 @@ class DisqusService(object):
                 "created_at": the UTC date this post was created in the format
                               %Y-%m-%dT%H:%M (the current time will be used by
                               default)
-                "author_url": the author's homepage; and "ip_address", their 
-                              IP address.
+                "author_url": the author's homepage
+                "ip_address": the author's IP address
         
         Action: Creates a new post on the thread. Does not check against spam
                 filters or ban list. This is intended to allow automated
@@ -88,13 +93,34 @@ class DisqusService(object):
         if parent_post:
             params["parent_post"] = parent_post
         if created_at:
-            if type(created_at) == type(datetime):
-                params["created_at"] = date_to_string(created_at)
+            params["created_at"] = date_to_string(created_at)
         if author_url:
             params["author_url"] = author_url
+        if ip_address:
+            params["ip_address"] = ip_address
         resp = self._http_request("create_post", params)
         return self._decode_post(forum, thread, resp)
 
+    def moderate_post(self, post_id, action):
+        """
+        Introduced in API 1.1
+        
+        Key: User Key
+        Arguments: "post_id"
+                   "action": name of action to be performed ("spam", "approve",
+                             or "kill").
+        
+        Result: Moderates post and returns modified version.
+        """
+        params = {
+            "post_id": post_id,
+            "action": action,
+        }
+        resp = self._http_request("moderate_post", params)
+        forum = self._decode_forum(resp["forum"])
+        thread = self._decode_thread(forum, resp["thread"])
+        return self._decode_post(forum, thread, resp)
+
     def get_forum_list(self):
         """
         Key: User Key
@@ -121,18 +147,35 @@ class DisqusService(object):
         """
         return self._http_request("get_forum_api_key", { "forum_id":forum.id })
 
-    def get_thread_list(self, forum):
+    def get_thread_list(self,
+                        forum,
+                        limit=None,
+                        start=None,
+                        category_id=None):
         """
         Key: Forum Key
-        Arguments: None.
+        Arguments:
+            Optional:
+                "limit": number of entries to be included in the response
+                         (default is 25)
+                "start": starting point for the query (default is 0)
+                "category_id": filter entries by their category.
         
         Result: A list of objects representing all threads belonging to the
                 given forum. See "Object Formats" for details on thread
                 objects.
         """
-        resp = self._http_request("get_thread_list", {
+        params = {
             "forum_api_key": forum.api_key,
-        })
+            "forum_id": forum.id,
+        }
+        if limit is not None:
+            params["limit"] = limit
+        if start is not None:
+            params["start"] = start
+        if category_id:
+            params["category_id"] = category_id
+        resp = self._http_request("get_thread_list", params)
         return [self._decode_thread(forum, t) for t in resp]
 
     def get_num_posts(self, forum, threads):
@@ -175,52 +218,91 @@ class DisqusService(object):
         })
         return self._decode_thread(forum, resp)
 
-    def get_thread_posts(self, forum, thread):
+    def get_thread_posts(self,
+                         forum,
+                         thread,
+                         limit=None,
+                         start=None,
+                         filter=None,
+                         exclude=None):
         """
         Key: Forum Key
-        Arguments: "thread_id": the ID of a thread belonging to the given 
-                                forum.
-
+        Arguments:
+            Required:
+                "thread_id": the ID of a thread belonging to the given forum.
+            Optional (introduced in API 1.1):
+                "limit": number of entries to be included in the response
+                         (default is 25)
+                "start": starting point for the query (default is 0)
+                "filter": type of entries to be returned ("new", "spam", or
+                          "killed")
+                "exclude": type of entries to be excluded from the response
+                           ("new", "spam", or "killed").
+        
         Result: A list of objects representing all posts belonging to the
                 given forum. See "Object Formats" for details on post objects. 
         """
-        resp = self._http_request("get_thread_posts", {
+        params = {
             "forum_api_key": forum.api_key,
             "thread_id": thread.id
-        })
+        }
+        if limit is not None:
+            params["limit"] = limit
+        if start is not None:
+            params["start"] = start
+        if filter:
+            params["filter"] = filter
+        if exclude:
+            params["exclude"] = exclude
+        resp = self._http_request("get_thread_posts", params)
         return [self._decode_post(forum, thread, post) for post in resp]
 
-    def thread_by_identifier(self, forum, title, identifier):
+    def thread_by_identifier(self,
+                             forum,
+                             title,
+                             identifier,
+                             category_id=None,
+                             create_on_fail=True):
         """
         Key: Forum Key
         Method: POST
-        Arguments: "title": the title of the thread to possibly be created
-                   "identifier": a string of your choosing (see Action).
-
-        Action: Create or retrieve a thread by an arbitrary identifying string
-                of your choice. For example, you could use your local
-                database's ID for the thread. This method allows you to
-                decouple thread identifiers from the URLs on which they might
-                be appear. (Disqus would normally use a thread's URL to
-                identify it, which is problematic when URLs do not uniquely 
-                identify a resource.) If no thread yet exists for the given
-                identifier (paired with the forum), one will be created.
-
+        Arguments:
+            Required:
+                "title": if thread does not exist, the method will create it
+                         with the specified title (can be an empty string if
+                         you are sure that the thread exists)
+                "identifier": unique value (per forum) for a thread that is
+                              used to get data even if permalink is changed.
+            Optional (introduced in API 1.1):
+                "category_id": if thread does not exist, the method will create
+                               it in the specified category
+                "create_on_fail": when False, a new thread will not be created
+                                  even if a matching thread is not found.
+        
+        Action: This method tries to find a thread by its identifier and title.
+                If there is no such thread and create_on_fail is undefined or
+                set to 1, the method creates it. If create_on_fail is set to 0,
+                this method will return an empty (but valid) response.
+        
         Result: An object with two keys:
                     "thread": which is the thread object corresponding to the
                               identifier
                     "created": which indicates whether the thread was created
                                as a result of this method call. If created, it
-                               will have the specified title. 
+                               will have the specified title.
         """
-        resp = self._http_request("thread_by_identifier", {
+        params = {
             "forum_api_key": forum.api_key,
             "title": title,
             "identifier": identifier,
-        })
+            "create_on_fail": 1 if create_on_fail else 0,
+        }
+        if category_id:
+            params["category_id"] = category_id
+        resp = self._http_request("thread_by_identifier", params)
         return {
-            "thread": self._decode_thread(forum, dct["thread"]),
-            "created": dct["created"],
+            "thread": self._decode_thread(forum, resp["thread"]),
+            "created": resp["created"],
         }
 
     def update_thread(self, 
@@ -265,6 +347,8 @@ class DisqusService(object):
         if user_key_required:
             data["user_api_key"] = self.get_user_api_key()
 
+        data["api_version"] = self.api_version
+
         method = REQUEST_METHODS[method_name]
 
         con = httplib.HTTPConnection(HOST)
@@ -362,6 +446,7 @@ class DisqusService(object):
             is_anonymous=dct.get("is_anonymous"),
             anonymous_author=self._decode_anonymous_author(dct.get("anonymous_author")),
             author=self._decode_author(dct.get("author")),
+            has_been_moderated=dct.get("has_been_moderated"),
         )
 
     def _decode_author(self, dct):
@@ -552,7 +637,8 @@ class Post(object):
                  shown,
                  is_anonymous=False,
                  anonymous_author=None,
-                 author=None):
+                 author=None,
+                 has_been_moderated=False):
         """
         id: a unique alphanumeric string identifying this Post object.
         forum: the id for the forum this post belongs to.
@@ -582,6 +668,7 @@ class Post(object):
         else:
             self.author = author
             self.anonymous_author = None
+        self.has_been_moderated = has_been_moderated
     
     def _get_parent_post(self):
         if self._parent_post_id and self._parent_post is None:
